@@ -1,5 +1,8 @@
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE Rank2Types            #-}
+{-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE TypeFamilies          #-}
 {-# LANGUAGE UndecidableInstances  #-}
 -----------------------------------------------------------------------------
@@ -18,7 +21,7 @@ module Diagrams.Combinators
 
          withEnvelope, withTrace
        , phantom, strut
-       , pad
+       , pad, frame
        , extrudeEnvelope, intrudeEnvelope
 
          -- * Binary operations
@@ -29,22 +32,35 @@ module Diagrams.Combinators
          -- * n-ary operations
        , appends
        , position, decorateTrail, decorateLocatedTrail, decoratePath
-       , cat, cat', CatOpts(..), CatMethod(..)
+       , cat, cat'
+       , CatOpts(_catMethod, _sep), catMethod, sep
+       , CatMethod(..)
 
        ) where
 
+import           Data.Typeable
+
+import           Control.Lens          (Lens', generateSignatures, lensField,
+                                        lensRules, makeLensesWith, (%~), (&),
+                                        (.~), (^.), _Wrapping)
 import           Data.AdditiveGroup
-import           Data.AffineSpace   ((.+^))
+import           Data.AffineSpace      ((.+^))
 import           Data.Default.Class
+import           Data.Monoid.Deletable (toDeletable)
+import           Data.Monoid.MList     (inj)
+#if __GLASGOW_HASKELL__ < 707
+import           Data.Proxy
+#endif
 import           Data.Semigroup
+import qualified Data.Tree.DUAL        as D
 import           Data.VectorSpace
 
 import           Diagrams.Core
+import           Diagrams.Core.Types   (QDiagram (QD))
 import           Diagrams.Located
 import           Diagrams.Path
-import           Diagrams.Segment   (Segment (..), straight)
-import           Diagrams.Trail     (Trail, trailVertices)
-import           Diagrams.TrailLike (fromOffsets)
+import           Diagrams.Segment      (straight)
+import           Diagrams.Trail        (Trail, trailVertices)
 import           Diagrams.Util
 
 ------------------------------------------------------------
@@ -53,6 +69,17 @@ import           Diagrams.Util
 
 -- | Use the envelope from some object as the envelope for a
 --   diagram, in place of the diagram's default envelope.
+--
+--   <<diagrams/src_Diagrams_Combinators_withEnvelopeEx.svg#diagram=withEnvelopeEx&width=300>>
+--
+--   > sqNewEnv =
+--   >     circle 1 # fc green
+--   >     |||
+--   >     (    c # dashingG [0.1,0.1] 0 # lc white
+--   >       <> square 2 # withEnvelope (c :: D R2) # fc blue
+--   >     )
+--   > c = circle 0.8
+--   > withEnvelopeEx = sqNewEnv # centerXY # pad 1.5
 withEnvelope :: (HasLinearMap (V a), Enveloped a, Monoid' m)
            => a -> QDiagram b (V a) m -> QDiagram b (V a) m
 withEnvelope = setEnvelope . getEnvelope
@@ -65,8 +92,8 @@ withTrace = setTrace . getTrace
 
 -- | @phantom x@ produces a \"phantom\" diagram, which has the same
 --   envelope and trace as @x@ but produces no output.
-phantom :: (Backend b (V a), Enveloped a, Traced a, Monoid' m) => a -> QDiagram b (V a) m
-phantom a = mkQD nullPrim (getEnvelope a) (getTrace a) mempty mempty
+phantom :: (Backend b (V a), Typeable (V a), Enveloped a, Traced a, Monoid' m) => a -> QDiagram b (V a) m
+phantom a = QD $ D.leafU ((inj . toDeletable . getEnvelope $ a) <> (inj . toDeletable . getTrace $ a))
 
 -- | @pad s@ \"pads\" a diagram, expanding its envelope by a factor of
 --   @s@ (factors between 0 and 1 can be used to shrink the envelope).
@@ -80,6 +107,16 @@ pad :: ( Backend b v
     => Scalar v -> QDiagram b v m -> QDiagram b v m
 pad s d = withEnvelope (d # scale s) d
 
+-- | @frame s@ increases the envelope of a diagram by and absolute amount @s@,
+--   s is in the local units of the diagram. This function is similar to @pad@,
+--   only it takes an absolute quantity and pre-centering should not be
+--   necessary.
+frame :: ( Backend b v, InnerSpace v, OrderedField (Scalar v), Monoid' m)
+        => Scalar v -> QDiagram b v m -> QDiagram b v m
+frame s d = setEnvelope (onEnvelope t (d^.envelope)) d
+  where
+    t f = \x -> f x + s
+
 -- | @strut v@ is a diagram which produces no output, but with respect
 --   to alignment and envelope acts like a 1-dimensional segment
 --   oriented along the vector @v@, with local origin at its
@@ -87,12 +124,17 @@ pad s d = withEnvelope (d # scale s) d
 --   with a nonempty trace see 'strutR2', 'strutX', and 'strutY' from
 --   "Diagrams.TwoD.Combinators".) Useful for manually creating
 --   separation between two diagrams.
-strut :: ( Backend b v, InnerSpace v
+--
+--   <<diagrams/src_Diagrams_Combinators_strutEx.svg#diagram=strutEx&width=300>>
+--
+--   > strutEx = (circle 1 ||| strut unitX ||| circle 1) # centerXY # pad 1.1
+strut :: ( Backend b v, Typeable v
+         , InnerSpace v
          , OrderedField (Scalar v)
          , Monoid' m
          )
       => v -> QDiagram b v m
-strut v = mkQD nullPrim env mempty mempty mempty
+strut v = QD $ D.leafU (inj . toDeletable $ env)
   where env = translate ((-0.5) *^ v) . getEnvelope $ straight v
   -- note we can't use 'phantom' here because it tries to construct a
   -- trace as well, and segments do not have a trace in general (only
@@ -135,10 +177,10 @@ deformEnvelope
   :: ( Ord (Scalar v), Num (Scalar v), AdditiveGroup (Scalar v)
      , Floating (Scalar v), HasLinearMap v, InnerSpace v, Monoid' m )
   => (Scalar v) -> v -> QDiagram b v m -> QDiagram b v m
-deformEnvelope s v d = setEnvelope (inEnvelope deform $ getEnvelope d) d
+deformEnvelope s v d = setEnvelope (getEnvelope d & _Wrapping Envelope %~ deformE) d
   where
-    deform = Option . fmap deform' . getOption
-    deform' env v'
+    deformE = Option . fmap deformE' . getOption
+    deformE' env v'
         | dot > 0 = Max $ getMax (env v') + (dot * s) / magnitude v'
         | otherwise = env v'
       where
@@ -167,7 +209,7 @@ infixl 6 `beneath`
 --   identity element, in which case the second object is returned
 --   unchanged).
 --
---   <<diagrams/besideEx.svg#diagram=besideEx&height=200>>
+--   <<diagrams/src_Diagrams_Combinators_besideEx.svg#diagram=besideEx&height=200>>
 --
 --   > besideEx = beside (r2 (20,30))
 --   >                   (circle 1 # fc orange)
@@ -203,12 +245,24 @@ beside v d1 d2 = d1 <> juxtapose v d1 d2
 --   @x@ in the corresponding direction.  Note that each object in
 --   @ys@ is positioned beside @x@ /without/ reference to the other
 --   objects in @ys@, so this is not the same as iterating 'beside'.
+--
+--   <<diagrams/src_Diagrams_Combinators_appendsEx.svg#diagram=appendsEx&width=200>>
+--
+--   > appendsEx = appends c (zip (iterateN 6 (rotateBy (1/6)) unitX) (repeat c))
+--   >             # centerXY # pad 1.1
+--   >   where c = circle 1
 appends :: (Juxtaposable a, Monoid' a) => a -> [(V a,a)] -> a
 appends d1 apps = d1 <> mconcat (map (\(v,d) -> juxtapose v d1 d) apps)
 
 -- | Position things absolutely: combine a list of objects
 --   (e.g. diagrams or paths) by assigning them absolute positions in
 --   the vector space of the combined object.
+--
+--   <<diagrams/src_Diagrams_Combinators_positionEx.svg#diagram=positionEx&height=300>>
+--
+--   > positionEx = position (zip (map mkPoint [-3, -2.8 .. 3]) (repeat dot))
+--   >   where dot       = circle 0.2 # fc black
+--   >         mkPoint x = p2 (x,x^2)
 position :: (HasOrigin a, Monoid' a) => [(Point (V a), a)] -> a
 position = mconcat . map (uncurry moveTo)
 
@@ -257,36 +311,48 @@ data CatMethod = Cat     -- ^ Normal catenation: simply put diagrams
                          --   of separation, diagrams may overlap.
 
 -- | Options for 'cat''.
-data CatOpts v = CatOpts { catMethod       :: CatMethod
-                             -- ^ Which 'CatMethod' should be used:
-                             --   normal catenation (default), or
-                             --   distribution?
-                         , sep             :: Scalar v
-                             -- ^ How much separation should be used
-                             --   between successive diagrams
-                             --   (default: 0)?  When @catMethod =
-                             --   Cat@, this is the distance between
-                             --   /envelopes/; when @catMethod =
-                             --   Distrib@, this is the distance
-                             --   between /origins/.
-                         , catOptsvProxy__ :: Proxy v
-                             -- ^ This field exists solely to aid type inference;
-                             --   please ignore it.
+data CatOpts v = CatOpts { _catMethod       :: CatMethod
+                         , _sep             :: Scalar v
+                         , _catOptsvProxy__ :: Proxy v
                          }
 
 -- The reason the proxy field is necessary is that without it,
 -- altering the sep field could theoretically change the type of a
--- CatOpts record.  This causes problems when writing an expression
--- like @with { sep = 10 }@, because knowing the type of the whole
+-- CatOpts record.  This causes problems when using record update, as
+-- in @with { _sep = 10 }@, because knowing the type of the whole
 -- expression does not tell us anything about the type of @with@, and
 -- therefore the @Num (Scalar v)@ constraint cannot be satisfied.
--- Adding the Proxy field constrains the type of @with@ in @with {sep
--- = 10}@ to be the same as the type of the whole expression.
+-- Adding the Proxy field constrains the type of @with@ in @with {_sep
+-- = 10}@ to be the same as the type of the whole expression.  Note
+-- this is not a problem when using the 'sep' lens, as its type is
+-- more restricted.
+
+makeLensesWith
+  ( lensRules
+    -- don't make a lens for the proxy field
+    & lensField .~ (\label ->
+        case label of
+          "_catOptsvProxy__" -> Nothing
+          _ -> Just (drop 1 label)
+        )
+    & generateSignatures .~ False
+  )
+  ''CatOpts
+
+-- | Which 'CatMethod' should be used:
+--   normal catenation (default), or distribution?
+catMethod :: forall v. Lens' (CatOpts v) CatMethod
+
+-- | How much separation should be used between successive diagrams
+--   (default: 0)?  When @catMethod = Cat@, this is the distance between
+--   /envelopes/; when @catMethod = Distrib@, this is the distance
+--   between /origins/.
+sep :: forall v. Lens' (CatOpts v) (Scalar v)
 
 instance Num (Scalar v) => Default (CatOpts v) where
-  def = CatOpts { catMethod       = Cat
-                , sep             = 0
-                , catOptsvProxy__ = Proxy
+  def = CatOpts { _catMethod       = Cat
+                , _sep             = 0
+                , _catOptsvProxy__ = Proxy
                 }
 
 -- | @cat v@ positions a list of objects so that their local origins
@@ -315,18 +381,18 @@ cat v = cat' v def
 --     default is 0.
 --
 --   'CatOpts' is an instance of 'Default', so 'with' may be used for
---   the second argument, as in @cat' (1,2) with {sep = 2}@.
+--   the second argument, as in @cat' (1,2) (with & sep .~ 2)@.
 --
---   Note that @cat' v with {catMethod = Distrib} === mconcat@
+--   Note that @cat' v (with & catMethod .~ Distrib) === mconcat@
 --   (distributing with a separation of 0 is the same as
 --   superimposing).
 cat' :: ( Juxtaposable a, Monoid' a, HasOrigin a
         , InnerSpace (V a), OrderedField (Scalar (V a))
         )
      => V a -> CatOpts (V a) -> [a] -> a
-cat' v (CatOpts { catMethod = Cat, sep = s }) = foldB comb mempty
+cat' v (CatOpts { _catMethod = Cat, _sep = s }) = foldB comb mempty
   where comb d1 d2 = d1 <> (juxtapose v d1 d2 # moveOriginBy vs)
         vs = s *^ normalized (negateV v)
 
-cat' v (CatOpts { catMethod = Distrib, sep = s }) =
+cat' v (CatOpts { _catMethod = Distrib, _sep = s }) =
   position . zip (iterate (.+^ (s *^ normalized v)) origin)
