@@ -26,41 +26,35 @@ module Diagrams.CubicSpline
 import           Diagrams.Core
 import           Diagrams.Core.Points
 import           Diagrams.CubicSpline.Internal
-import           Diagrams.Located              (at)
-import           Diagrams.Located
-import           Diagrams.Path
+import           Diagrams.Located              (Located, at, mapLoc)
 import           Diagrams.Segment
 import           Diagrams.Trail
 import           Diagrams.TrailLike            (TrailLike (..))
 
--- for e.g. the Fractional (Double, Double) instance
-import           Data.NumInstances.Tuple       ()
-
-import           Control.Newtype
-import           Data.Semigroup
+import           Data.AffineSpace.Point
 import           Data.VectorSpace
 
 -- | Construct a spline path-like thing of cubic segments from a list of
 --   vertices, with the first vertex as the starting point.  The first
 --   argument specifies whether the path should be closed.
 --
---   <<diagrams/cubicSplineEx.svg#diagram=cubicSplineEx&width=600>>
+--   <<diagrams/src_Diagrams_CubicSpline_cubicSplineEx.svg#diagram=cubicSplineEx&width=600>>
 --
 --   > pts = map p2 [(0,0), (2,3), (5,-2), (-4,1), (0,3)]
---   > dot = circle 0.2 # fc blue # lw 0
+--   > dot = circle 0.2 # fc blue # lw none
 --   > mkPath closed = position (zip pts (repeat dot))
---   >              <> cubicSpline closed pts # lw 0.05
+--   >              <> cubicSpline closed pts
 --   > cubicSplineEx = (mkPath False ||| strutX 2 ||| mkPath True)
 --   >               # centerXY # pad 1.1
 --
 --   For more information, see <http://mathworld.wolfram.com/CubicSpline.html>.
 cubicSpline :: (TrailLike t, Fractional (V t)) => Bool -> [Point (V t)] -> t
-cubicSpline c [] = trailLike . closeIf c $ emptyLine `at` origin
-cubicSpline c ps = flattenBeziers . map f . solveCubicSplineCoefficients c . map unpack $ ps
+cubicSpline closed [] = trailLike . closeIf closed $ emptyLine `at` origin
+cubicSpline closed ps = flattenBeziers . map f . solveCubicSplineCoefficients closed . map unPoint $ ps
   where
     f [a,b,c,d] = [a, (3*a+b)/3, (3*a+2*b+c)/3, a+b+c+d]
     flattenBeziers bs@((b:_):_)
-      = trailLike . closeIf c $ lineFromSegments (map bez bs) `at` P b
+      = trailLike . closeIf closed $ lineFromSegments (map bez bs) `at` P b
     bez [a,b,c,d] = bezier3 (b - a) (c - a) (d - a)
 
 closeIf :: (InnerSpace v, OrderedField (Scalar v))
