@@ -1,6 +1,7 @@
 {-# LANGUAGE DeriveFunctor             #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE ScopedTypeVariables       #-}
+{-# LANGUAGE TemplateHaskell           #-}
 {-# LANGUAGE TypeFamilies              #-}
 {-# LANGUAGE ViewPatterns              #-}
 
@@ -20,7 +21,7 @@ module Diagrams.TwoD.Polygons(
         -- * Polygons
           PolyType(..)
         , PolyOrientation(..)
-        , PolygonOpts(..)
+        , PolygonOpts(..), polyType, polyOrient, polyCenter
 
         , polygon
         , polyTrail
@@ -44,27 +45,26 @@ module Diagrams.TwoD.Polygons(
 
     ) where
 
+import           Control.Lens            (Lens', generateSignatures, lensRules,
+                                          makeLensesWith, (.~), (^.), view)
 import           Control.Monad           (forM, liftM)
 import           Control.Monad.ST        (ST, runST)
 import           Data.Array.ST           (STUArray, newArray, readArray,
                                           writeArray)
-import qualified Data.Foldable           as F
 import           Data.List               (maximumBy, minimumBy)
-import qualified Data.List.NonEmpty      as NEL
 import           Data.Maybe              (catMaybes)
 import           Data.Monoid             (mconcat, mempty)
 import           Data.Ord                (comparing)
 
 import           Data.AffineSpace        ((.+^), (.-.))
 import           Data.Default.Class
-import           Data.VectorSpace        (magnitude, normalized, project, (<.>),
-                                          (^*))
+import           Data.VectorSpace
 
+import           Diagrams.Angle
 import           Diagrams.Core
 import           Diagrams.Located
 import           Diagrams.Path
 import           Diagrams.Points         (centroid)
-import           Diagrams.Segment
 import           Diagrams.Trail
 import           Diagrams.TrailLike
 import           Diagrams.TwoD.Transform
@@ -73,7 +73,7 @@ import           Diagrams.TwoD.Vector    (leftTurn, unitX, unitY, unit_Y)
 import           Diagrams.Util           (tau, ( # ))
 
 -- | Method used to determine the vertices of a polygon.
-data PolyType = forall a. Angle a => PolyPolar [a] [Double]
+data PolyType = PolyPolar [Angle] [Double]
                 -- ^ A \"polar\" polygon.
                 --
                 --   * The first argument is a list of /central/
@@ -90,7 +90,7 @@ data PolyType = forall a. Angle a => PolyPolar [a] [Double]
                 --   circle) can be constructed using a second
                 --   argument of @(repeat r)@.
 
-              | forall a. Angle a => PolySides [a] [Double]
+              | PolySides [Angle] [Double]
                 -- ^ A polygon determined by the distance between
                 --   successive vertices and the angles formed by
                 --   each three successive vertices.  In other
@@ -138,20 +138,24 @@ data PolyOrientation = NoOrient     -- ^ No special orientation; the first
 
 -- | Options for specifying a polygon.
 data PolygonOpts = PolygonOpts
-                   { polyType   :: PolyType
-                     -- ^ Specification for the polygon's vertices.
-
-                   , polyOrient :: PolyOrientation
-                     -- ^ Should a rotation be applied to the
-                     --   polygon in order to orient it in a
-                     --   particular way?
-
-                   , polyCenter :: P2
-                     -- ^ Should a translation be applied to the
-                     --   polygon in order to place the center at a
-                     --   particular location?
+                   { _polyType   :: PolyType
+                   , _polyOrient :: PolyOrientation
+                   , _polyCenter :: P2
                    }
 
+makeLensesWith (generateSignatures .~ False $ lensRules) ''PolygonOpts
+
+-- | Specification for the polygon's vertices.
+polyType :: Lens' PolygonOpts PolyType
+
+-- | Should a rotation be applied to the polygon in order to orient it in a
+--   particular way?
+polyOrient :: Lens' PolygonOpts PolyOrientation
+
+-- | Should a translation be applied to the polygon in order to place the center
+--   at a particular location?
+polyCenter :: Lens' PolygonOpts P2
+
 -- | The default polygon is a regular pentagon of radius 1, centered
 --   at the origin, aligned to the x-axis.
 instance Default PolygonOpts where
@@ -161,11 +165,11 @@ instance Default PolygonOpts where
 polyTrail :: PolygonOpts -> Located (Trail R2)
 polyTrail po = transform ori tr
     where
-        tr = case polyType po of
+        tr = case po^.polyType of
             PolyPolar ans szs -> polyPolarTrail ans szs
             PolySides ans szs -> polySidesTrail ans szs
             PolyRegular n r   -> polyRegularTrail n r
-        ori = case polyOrient po of
+        ori = case po^.polyOrient of
             OrientH      -> orient unit_Y tr
             OrientV      -> orient unitX  tr
             OrientTo v   -> orient v      tr
@@ -177,7 +181,7 @@ polygon = trailLike . polyTrail
 
 -- | Generate the located trail of a polygon specified by polar data
 --   (central angles and radii). See 'PolyPolar'.
-polyPolarTrail :: Angle a => [a] -> [Double] -> Located (Trail R2)
+polyPolarTrail :: [Angle] -> [Double] -> Located (Trail R2)
 polyPolarTrail [] _ = emptyTrail `at` origin
 polyPolarTrail _ [] = emptyTrail `at` origin
 polyPolarTrail ans (r:rs) = tr `at` p1
@@ -186,16 +190,16 @@ polyPolarTrail ans (r:rs) = tr `at` p1
     tr = closeTrail . trailFromVertices $
            zipWith
              (\a l -> rotate a . scale l $ p2 (1,0))
-             (scanl (+) 0 ans)
+             (scanl (^+^) zeroV ans)
              (r:rs)
 
 -- | Generate the vertices of a polygon specified by side length and
 --   angles, and a starting point for the trail such that the origin
 --   is at the centroid of the vertices.  See 'PolySides'.
-polySidesTrail :: Angle a => [a] -> [Double] -> Located (Trail R2)
+polySidesTrail :: [Angle] -> [Double] -> Located (Trail R2)
 polySidesTrail ans ls = tr `at` (centroid ps # scale (-1))
   where
-    ans'    = scanl (+) 0 ans
+    ans'    = scanl (^+^) zeroV ans
     offsets = zipWith rotate ans' (map (unitY ^*) ls)
     ps      = scanl (.+^) origin offsets
     tr      = closeTrail . trailFromOffsets $ offsets
@@ -203,7 +207,7 @@ polySidesTrail ans ls = tr `at` (centroid ps # scale (-1))
 -- | Generate the vertices of a regular polygon.  See 'PolyRegular'.
 polyRegularTrail :: Int -> Double -> Located (Trail R2)
 polyRegularTrail n r = polyPolarTrail
-                         (take (n-1) . repeat $ (tau::Rad) / fromIntegral n)
+                         (take (n-1) . repeat $ fullTurn ^/ fromIntegral n)
                          (repeat r)
 
 -- | Generate a transformation to orient a trail.  @orient v t@
@@ -219,18 +223,22 @@ orientPoints v xs = rotation a
     (n1,x,n2) = maximumBy (comparing (distAlong v . sndOf3))
                   (zip3 (tail (cycle xs)) xs (last xs : init xs))
     distAlong w ((.-. origin) -> p) = signum (w <.> p) * magnitude (project w p)
-    sndOf3 (_,x,_) = x
-    a = minimumBy (comparing abs) . map (angleFromNormal . (.-. x)) $ [n1,n2]
+    sndOf3 (_,b,_) = b
+    a :: Angle
+    a = minimumBy (comparing $ abs . view rad)
+        . map (angleFromNormal . (.-. x)) $ [n1,n2]
     v' = normalized v
+    angleFromNormal :: R2 -> Angle
     angleFromNormal o
       | leftTurn o' v' = phi
-      | otherwise      = negate phi
+      | otherwise      = negateV phi
       where
         o' = normalized o
         theta = acos (v' <.> o')
+        phi :: Angle
         phi
-          | theta <= tau/4 = Rad $ tau/4 - theta
-          | otherwise      = Rad $ theta - tau/4
+          | theta <= tau/4 = tau/4 - theta @@ rad
+          | otherwise      = theta - tau/4 @@ rad
 
 ------------------------------------------------------------
 -- Function graphs
