@@ -2,6 +2,11 @@
 {-# LANGUAGE FlexibleInstances    #-}
 {-# LANGUAGE UndecidableInstances #-}
 
+{-# OPTIONS_GHC -fno-warn-orphans #-}
+-- Orphan Traced instances for Segment Closed R2 and FixedSegment R2.
+-- They can't go in Traced; but they shouldn't really go in
+-- Diagrams.Segment either because we only have Traced instances for
+-- the special case of R2.
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.TwoD.Segment
@@ -19,11 +24,9 @@ module Diagrams.TwoD.Segment where
 import           Control.Applicative     (liftA2)
 
 import           Data.AffineSpace
-import           Data.Monoid.Inf         hiding (minimum)
 import           Data.VectorSpace
 
 import           Diagrams.Core
-import           Diagrams.Core.Trace
 
 import           Diagrams.Located
 import           Diagrams.Parametric
@@ -34,6 +37,10 @@ import           Diagrams.TwoD.Types
 import           Diagrams.TwoD.Vector
 import           Diagrams.Util
 
+{- All instances of Traced should maintain the invariant that the list of
+   traces is sorted in increasing order.
+-}
+
 instance Traced (Segment Closed R2) where
   getTrace = getTrace . mkFixedSeg . (`at` origin)
 
@@ -64,14 +71,13 @@ instance Traced (FixedSegment R2) where
     let
       v0     = p0' .-. p0
       det    = perp v1 <.> v0
-      perp v = rotateBy (-1/4) v
       p      = p1 .-. p0
       t0     = (perp v1 <.> p) / det
       t1     = (perp v0 <.> p) / det
     in
       if det == 0 || t0 < 0 || t0 > 1
-        then Infinity
-        else Finite t1
+        then mkSortedList []
+        else mkSortedList [t1]
 
 {- To do intersection of a line with a cubic Bezier, we first rotate
    and scale everything so that the line has parameters (origin, unitX);
@@ -87,7 +93,7 @@ instance Traced (FixedSegment R2) where
     let
       bez'@(FCubic x1 c1 c2 x2) =
         bez # moveOriginTo p1
-            # rotateBy (negate (direction v1))
+            # rotate (negateV (direction v1))
             # scale (1/magnitude v1)
       [y0,y1,y2,y3] = map (snd . unp2) [x1,c1,c2,x2]
       a  = -y0 + 3*y1 - 3*y2 + y3
@@ -97,7 +103,5 @@ instance Traced (FixedSegment R2) where
       ts = filter (liftA2 (&&) (>= 0) (<= 1)) (cubForm a b c d)
       xs = map (fst . unp2 . atParam bez') ts
     in
-      case xs of
-        [] -> Infinity
-        _  -> Finite (minimum xs)
+      mkSortedList xs
 
